/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.MedorEvaluator;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.genclass.RdbFieldDesc;
import org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassProp;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;


// Import clauses required by the object BnUser
import jorm.BnUserAccessor;
import jorm.BnUserPKPNG;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class BnUserMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
	, PNameGetter, jorm.BnUserPKPNG
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String bnAuthRoles_NAME = "bnAuthRoles";
	protected static final String bnProjects_NAME = "bnProjects";
	protected static final String bnProperties_NAME = "bnProperties";
	protected static final String bnRoles_NAME = "bnRoles";
	protected static final String createdBnAgents_NAME = "createdBnAgents";
	protected static final String creationDate_NAME = "creationDate";
	protected static final String email_NAME = "email";
	protected static final String id_NAME = "id";
	protected static final String jabber_NAME = "jabber";
	protected static final String modificationDate_NAME = "modificationDate";
	protected static final String name_NAME = "name";
	protected static final String password_NAME = "password";
	//Hidden fields

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.BnUser";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder bnAuthRolesPnc;
	PClassMapping bnAuthRolesGCM;
	public PNameCoder bnProjectsPnc;
	PClassMapping bnProjectsGCM;
	public PNameCoder bnPropertiesPnc;
	PClassMapping bnPropertiesGCM;
	public PNameCoder bnRolesPnc;
	PClassMapping bnRolesGCM;
	public PNameCoder createdBnAgentsPnc;
	PClassMapping createdBnAgentsGCM;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public BnUserMapping() {
 	}
	public PNameCoder getBnAuthRolesPNameCoder() {
	    return bnAuthRolesPnc;
	}
	public PNameCoder getBnProjectsPNameCoder() {
	    return bnProjectsPnc;
	}
	public PNameCoder getBnPropertiesPNameCoder() {
	    return bnPropertiesPnc;
	}
	public PNameCoder getBnRolesPNameCoder() {
	    return bnRolesPnc;
	}
	public PNameCoder getCreatedBnAgentsPNameCoder() {
	    return createdBnAgentsPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (bnAuthRoles_NAME.equals(fn)) {
			bnAuthRolesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnProjects_NAME.equals(fn)) {
			bnProjectsPnc = (PNamingContext) pnc;
			return;
		}
		if (bnProperties_NAME.equals(fn)) {
			bnPropertiesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnRoles_NAME.equals(fn)) {
			bnRolesPnc = (PNamingContext) pnc;
			return;
		}
		if (createdBnAgents_NAME.equals(fn)) {
			createdBnAgentsPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		if (bnAuthRoles_NAME.equals(fn)) {
			bnAuthRolesGCM = pcm;
			return;
		}
		if (bnProjects_NAME.equals(fn)) {
			bnProjectsGCM = pcm;
			return;
		}
		if (bnProperties_NAME.equals(fn)) {
			bnPropertiesGCM = pcm;
			return;
		}
		if (bnRoles_NAME.equals(fn)) {
			bnRolesGCM = pcm;
			return;
		}
		if (createdBnAgents_NAME.equals(fn)) {
			createdBnAgentsGCM = pcm;
			return;
		}
		throw new UnsupportedOperationException("This class does not contain a " + fn + " field referencing a generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.BnUser",
				SUPER_TYPES);
		setPType(ptype);
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}

         // Initialization of the gen class /jorm.BnUser/bnAuthRoles
        FieldDesc[] fds = null;
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnUser_id", "id", PTypeSpace.STRING, "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnAuthRolesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnAuthRolesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnAuthRole_id", "id", PTypeSpace.STRING, "BnAuthRole_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnAuthRolesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnAuthRolesGCM).setTableName("BNAUTHROLE_BNUSER");
			((RdbGenClassProp) bnAuthRolesGCM).setColocated(false);
			((RdbGenClassProp) bnAuthRolesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnAuthRolesGCM).setReadOnly(false);
			((RdbGenClassProp) bnAuthRolesGCM).setTypeConverter(adapter);
        String[] bnAuthRolesGCDef = {"java.util.Collection"};
        String[][] bnAuthRolesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnAuthRolesGCDef,
            "jorm.BnAuthRole",
            bnAuthRolesICDef);
        ((GenClassProp) bnAuthRolesGCM).setPType(ptype);
        bnAuthRolesPnc.setPType(ptype);
        bnAuthRolesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnUser/bnProjects
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnUser_id", "id", PTypeSpace.STRING, "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnProjectsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnProjectsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnProjectsGCM).setElemDesc(fds);
			((RdbGenClassProp) bnProjectsGCM).setTableName("BNUSER_BNPROJECT");
			((RdbGenClassProp) bnProjectsGCM).setColocated(false);
			((RdbGenClassProp) bnProjectsGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnProjectsGCM).setReadOnly(true);
			((RdbGenClassProp) bnProjectsGCM).setTypeConverter(adapter);
        String[] bnProjectsGCDef = {"java.util.Collection"};
        String[][] bnProjectsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnProjectsGCDef,
            "jorm.BnProject",
            bnProjectsICDef);
        ((GenClassProp) bnProjectsGCM).setPType(ptype);
        bnProjectsPnc.setPType(ptype);
        bnProjectsGCM.init(m, null);

         // Initialization of the gen class /jorm.BnUser/createdBnAgents
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnUser_id", "id", PTypeSpace.STRING, "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) createdBnAgentsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) createdBnAgentsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnAgent_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) createdBnAgentsGCM).setElemDesc(fds);
			((RdbGenClassProp) createdBnAgentsGCM).setTableName("BnAgent");
			((RdbGenClassProp) createdBnAgentsGCM).setColocated(true);
			((RdbGenClassProp) createdBnAgentsGCM).setColocatedMaster(false);
			((RdbGenClassProp) createdBnAgentsGCM).setReadOnly(true);
			((RdbGenClassProp) createdBnAgentsGCM).setTypeConverter(adapter);
        String[] createdBnAgentsGCDef = {"java.util.Collection"};
        String[][] createdBnAgentsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            createdBnAgentsGCDef,
            "jorm.BnAgent",
            createdBnAgentsICDef);
        ((GenClassProp) createdBnAgentsGCM).setPType(ptype);
        createdBnAgentsPnc.setPType(ptype);
        createdBnAgentsGCM.init(m, null);

         // Initialization of the gen class /jorm.BnUser/bnRoles
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnUser_id", "id", PTypeSpace.STRING, "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnRolesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnRolesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnRole_id", "id", PTypeSpace.STRING, "BnRole_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnRolesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnRolesGCM).setTableName("BNUSER_BNROLE");
			((RdbGenClassProp) bnRolesGCM).setColocated(false);
			((RdbGenClassProp) bnRolesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnRolesGCM).setReadOnly(true);
			((RdbGenClassProp) bnRolesGCM).setTypeConverter(adapter);
        String[] bnRolesGCDef = {"java.util.Collection"};
        String[][] bnRolesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnRolesGCDef,
            "jorm.BnRole",
            bnRolesICDef);
        ((GenClassProp) bnRolesGCM).setPType(ptype);
        bnRolesPnc.setPType(ptype);
        bnRolesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnUser/bnProperties
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnUser_id", "id", PTypeSpace.STRING, "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnPropertiesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnPropertiesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnUserProperty_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnPropertiesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnPropertiesGCM).setTableName("BnUserProperty");
			((RdbGenClassProp) bnPropertiesGCM).setColocated(true);
			((RdbGenClassProp) bnPropertiesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnPropertiesGCM).setReadOnly(true);
			((RdbGenClassProp) bnPropertiesGCM).setTypeConverter(adapter);
        String[] bnPropertiesGCDef = {"java.util.Collection"};
        String[][] bnPropertiesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnPropertiesGCDef,
            "jorm.BnUserProperty",
            bnPropertiesICDef);
        ((GenClassProp) bnPropertiesGCM).setPType(ptype);
        bnPropertiesPnc.setPType(ptype);
        bnPropertiesGCM.init(m, null);
 		//register the structures and dependencies
		init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.BnUser
			        	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "password", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "email", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "name", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "jabber", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, -1, -1), true, true, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "creationdate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUser", "modificationdate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	    		        // Initialization of the structures of the gen class /jorm.BnUser/bnAuthRoles
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNAUTHROLE_BNUSER", "BnAuthRole_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNAUTHROLE_BNUSER", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnUser/bnProjects
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNUSER_BNPROJECT", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNUSER_BNPROJECT", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnUser/createdBnAgents
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgent", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgent", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnUser/bnRoles
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNUSER_BNROLE", "BnRole_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNUSER_BNROLE", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnUser/bnProperties
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnUserProperty", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnUserProperty", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
						//dependencies
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnUserProperty");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnAuthRole");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnRole");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnAgent");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnProject");
		} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (bnAuthRoles_NAME.equals(fn))
				return bnAuthRolesPnc;
			if (bnProjects_NAME.equals(fn))
				return bnProjectsPnc;
			if (bnProperties_NAME.equals(fn))
				return bnPropertiesPnc;
			if (bnRoles_NAME.equals(fn))
				return bnRolesPnc;
			if (createdBnAgents_NAME.equals(fn))
				return createdBnAgentsPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			if (bnAuthRoles_NAME.equals(fn))
				return bnAuthRolesGCM;
			if (bnProjects_NAME.equals(fn))
				return bnProjectsGCM;
			if (bnProperties_NAME.equals(fn))
				return bnPropertiesGCM;
			if (bnRoles_NAME.equals(fn))
				return bnRolesGCM;
			if (createdBnAgents_NAME.equals(fn))
				return createdBnAgentsGCM;
			throw new UnsupportedOperationException("This class does not contain a " + fn + " field which would reference a generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.BnUserBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
			if(withSubType){
				//the list of fields belonging to the primary key
				String pk = "BnUser.id";
				queryToExecute = rdbcm.getRdbInheritanceQuery().getExtentQuery(mapper,prefetching, true, pk, this);	
				//System.out.println("query to execute:" + queryToExecute + " compared with the generated query: " + (prefetching ? prefetchedExtentQuery : extentQuery));
				beginIndex =  rdbcm.getRdbInheritanceQuery().getExtentFieldsNames(prefetching, true).size() + 1;
			}
			else{
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 7 + 1;
			}
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new BnUserResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <BnUser>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <BnUser>.");
        }
		return it;
	}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
					return false;
				}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeAbstract(pname.encodeAbstract(), null);
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
        PClassMapping gcm = null;
        bnAuthRolesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnAuthRoles_NAME, new String[]{"java.util.Collection"});
        bnAuthRolesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnAuthRoles_NAME, new String[]{"java.util.Collection"}, "jorm.BnAuthRole");
        gcm = bnAuthRolesGCM;
        bnProjectsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnProjects_NAME, new String[]{"java.util.Collection"});
        bnProjectsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnProjects_NAME, new String[]{"java.util.Collection"}, "jorm.BnProject");
        gcm = bnProjectsGCM;
        bnPropertiesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnProperties_NAME, new String[]{"java.util.Collection"});
        bnPropertiesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnProperties_NAME, new String[]{"java.util.Collection"}, "jorm.BnUserProperty");
        gcm = bnPropertiesGCM;
        bnRolesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnRoles_NAME, new String[]{"java.util.Collection"});
        bnRolesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnRoles_NAME, new String[]{"java.util.Collection"}, "jorm.BnRole");
        gcm = bnRolesGCM;
        createdBnAgentsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, createdBnAgents_NAME, new String[]{"java.util.Collection"});
        createdBnAgentsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, createdBnAgents_NAME, new String[]{"java.util.Collection"}, "jorm.BnAgent");
        gcm = createdBnAgentsGCM;
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		BnUserAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnUserAccessor)
			paccessor = (BnUserAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table BnUser : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteBnUserTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnUser]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertBnUserTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnUser]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateBnUserTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnUser]: problematic SQL query -> {" + sqlquery + "}");
                }
		}
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}

	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		BnUserAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnUserAccessor)
			paccessor = (BnUserAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            pstmt = connection.prepareStatement(readQuery);
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnUserPKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnUserPKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetCreationDate(adapter.getDate(resultSet, 1, null, null));
                paccessor.paSetEmail(adapter.getString(resultSet, 2, null));
                paccessor.paSetId(adapter.getString(resultSet, 3, null));
                paccessor.paSetJabber(adapter.getString(resultSet, 4, null));
                paccessor.paSetModificationDate(adapter.getDate(resultSet, 5, null, null));
                paccessor.paSetName(adapter.getString(resultSet, 6, null));
                paccessor.paSetPassword(adapter.getString(resultSet, 7, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@c50e46 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@a7b68a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProperties / join=null
                refname = bnPropertiesPnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessor.paSetBnProperties(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@10a4761 / cnId=3 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@c13b15 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnRoles / join=null
                refname = bnRolesPnc.decodeAbstract(getInstancePNG3(), resultSet);
                paccessor.paSetBnRoles(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1d4433f / cnId=1 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1dfc6e / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=createdBnAgents / join=null
                refname = createdBnAgentsPnc.decodeAbstract(getInstancePNG1(), resultSet);
                paccessor.paSetCreatedBnAgents(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@8b1a8f / cnId=4 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@de537 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnAuthRoles / join=null
                refname = bnAuthRolesPnc.decodeAbstract(getInstancePNG4(), resultSet);
                paccessor.paSetBnAuthRoles(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7caac5 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@43882a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProjects / join=null
                refname = bnProjectsPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessor.paSetBnProjects(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetDateField(creationDate_NAME, adapter.getDate(resultSet, 1, null, null));
                paccessorGen.paSetStringField(email_NAME, adapter.getString(resultSet, 2, null));
                paccessorGen.paSetStringField(id_NAME, adapter.getString(resultSet, 3, null));
                paccessorGen.paSetStringField(jabber_NAME, adapter.getString(resultSet, 4, null));
                paccessorGen.paSetDateField(modificationDate_NAME, adapter.getDate(resultSet, 5, null, null));
                paccessorGen.paSetStringField(name_NAME, adapter.getString(resultSet, 6, null));
                paccessorGen.paSetStringField(password_NAME, adapter.getString(resultSet, 7, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7caac5 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@43882a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProjects / join=null
                refname = bnProjectsPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessorGen.paSetRefField(bnProjects_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7caac5 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@43882a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProjects / join=null
                refname = bnProjectsPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessorGen.paSetRefField(bnProjects_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7caac5 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@43882a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProjects / join=null
                refname = bnProjectsPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessorGen.paSetRefField(bnProjects_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7caac5 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@43882a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProjects / join=null
                refname = bnProjectsPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessorGen.paSetRefField(bnProjects_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7caac5 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@12c4b66} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@43882a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@1ee148b / nd.nr.cn={id=id} / fieldName=bnProjects / join=null
                refname = bnProjectsPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessorGen.paSetRefField(bnProjects_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa);
	            return;
	        }
		    BnUserAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof BnUserAccessor)
			    paccessor = (BnUserAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
                paccessor.paSetBnAuthRoles(bnAuthRolesPnc.decodeAbstract(getInstancePNGPrefetchbnAuthRoles(), tuple), conn);
                paccessor.paSetBnProjects(bnProjectsPnc.decodeAbstract(getInstancePNGPrefetchbnProjects(), tuple), conn);
                paccessor.paSetBnProperties(bnPropertiesPnc.decodeAbstract(getInstancePNGPrefetchbnProperties(), tuple), conn);
                paccessor.paSetBnRoles(bnRolesPnc.decodeAbstract(getInstancePNGPrefetchbnRoles(), tuple), conn);
                paccessor.paSetCreatedBnAgents(createdBnAgentsPnc.decodeAbstract(getInstancePNGPrefetchcreatedBnAgents(), tuple), conn);
                paccessor.paSetCreationDate(tuple.getDate(1));
                paccessor.paSetEmail(tuple.getString(2));
                paccessor.paSetId(tuple.getString(3));
                paccessor.paSetJabber(tuple.getString(4));
                paccessor.paSetModificationDate(tuple.getDate(5));
                paccessor.paSetName(tuple.getString(6));
                paccessor.paSetPassword(tuple.getString(7));
            } else if (paccessorGen != null) {
                paccessorGen.paSetRefField(bnAuthRoles_NAME, bnAuthRolesPnc.decodeAbstract(getInstancePNGPrefetchbnAuthRoles(), tuple), conn);
                paccessorGen.paSetRefField(bnProjects_NAME, bnProjectsPnc.decodeAbstract(getInstancePNGPrefetchbnProjects(), tuple), conn);
                paccessorGen.paSetRefField(bnProperties_NAME, bnPropertiesPnc.decodeAbstract(getInstancePNGPrefetchbnProperties(), tuple), conn);
                paccessorGen.paSetRefField(bnRoles_NAME, bnRolesPnc.decodeAbstract(getInstancePNGPrefetchbnRoles(), tuple), conn);
                paccessorGen.paSetRefField(createdBnAgents_NAME, createdBnAgentsPnc.decodeAbstract(getInstancePNGPrefetchcreatedBnAgents(), tuple), conn);
                paccessorGen.paSetDateField(creationDate_NAME, tuple.getDate(1));
                paccessorGen.paSetStringField(email_NAME, tuple.getString(2));
                paccessorGen.paSetStringField(id_NAME, tuple.getString(3));
                paccessorGen.paSetStringField(jabber_NAME, tuple.getString(4));
                paccessorGen.paSetDateField(modificationDate_NAME, tuple.getDate(5));
                paccessorGen.paSetStringField(name_NAME, tuple.getString(6));
                paccessorGen.paSetStringField(password_NAME, tuple.getString(7));
            }
			((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT BnUser.id FROM BnUser WHERE BnUser.id = ?");
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnUserPKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnUserPKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}
    private static BnUser_bnAuthRolesPNGPrefetch singletonPNGPrefetchbnAuthRoles = null;

    private BnUser_bnAuthRolesPNGPrefetch getInstancePNGPrefetchbnAuthRoles() throws PException {
        if (singletonPNGPrefetchbnAuthRoles==null) {
            singletonPNGPrefetchbnAuthRoles = new BnUser_bnAuthRolesPNGPrefetch(bnAuthRolesPnc);
		}
		return singletonPNGPrefetchbnAuthRoles;
	}
    private static BnUser_bnProjectsPNGPrefetch singletonPNGPrefetchbnProjects = null;

    private BnUser_bnProjectsPNGPrefetch getInstancePNGPrefetchbnProjects() throws PException {
        if (singletonPNGPrefetchbnProjects==null) {
            singletonPNGPrefetchbnProjects = new BnUser_bnProjectsPNGPrefetch(bnProjectsPnc);
		}
		return singletonPNGPrefetchbnProjects;
	}
    private static BnUser_bnPropertiesPNGPrefetch singletonPNGPrefetchbnProperties = null;

    private BnUser_bnPropertiesPNGPrefetch getInstancePNGPrefetchbnProperties() throws PException {
        if (singletonPNGPrefetchbnProperties==null) {
            singletonPNGPrefetchbnProperties = new BnUser_bnPropertiesPNGPrefetch(bnPropertiesPnc);
		}
		return singletonPNGPrefetchbnProperties;
	}
    private static BnUser_bnRolesPNGPrefetch singletonPNGPrefetchbnRoles = null;

    private BnUser_bnRolesPNGPrefetch getInstancePNGPrefetchbnRoles() throws PException {
        if (singletonPNGPrefetchbnRoles==null) {
            singletonPNGPrefetchbnRoles = new BnUser_bnRolesPNGPrefetch(bnRolesPnc);
		}
		return singletonPNGPrefetchbnRoles;
	}
    private static BnUser_createdBnAgentsPNGPrefetch singletonPNGPrefetchcreatedBnAgents = null;

    private BnUser_createdBnAgentsPNGPrefetch getInstancePNGPrefetchcreatedBnAgents() throws PException {
        if (singletonPNGPrefetchcreatedBnAgents==null) {
            singletonPNGPrefetchcreatedBnAgents = new BnUser_createdBnAgentsPNGPrefetch(createdBnAgentsPnc);
		}
		return singletonPNGPrefetchcreatedBnAgents;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private static jorm.BnUserPKPNG singletonPNG2 = null;
    public jorm.BnUserPKPNG getInstancePNG2() throws PException {
        if (singletonPNG2==null) {
            singletonPNG2 = new  BnUserPNG2(bnPropertiesPnc, adapter);
		}
		return singletonPNG2;
	}

    private static jorm.BnUserPKPNG singletonPNG3 = null;
    public jorm.BnUserPKPNG getInstancePNG3() throws PException {
        if (singletonPNG3==null) {
            singletonPNG3 = new  BnUserPNG3(bnRolesPnc, adapter);
		}
		return singletonPNG3;
	}

    private static jorm.BnUserPKPNG singletonPNG1 = null;
    public jorm.BnUserPKPNG getInstancePNG1() throws PException {
        if (singletonPNG1==null) {
            singletonPNG1 = new  BnUserPNG1(createdBnAgentsPnc, adapter);
		}
		return singletonPNG1;
	}

    private static jorm.BnUserPKPNG singletonPNG4 = null;
    public jorm.BnUserPKPNG getInstancePNG4() throws PException {
        if (singletonPNG4==null) {
            singletonPNG4 = new  BnUserPNG4(bnAuthRolesPnc, adapter);
		}
		return singletonPNG4;
	}

    private static jorm.BnUserPKPNG singletonPNG5 = null;
    public jorm.BnUserPKPNG getInstancePNG5() throws PException {
        if (singletonPNG5==null) {
            singletonPNG5 = new  BnUserPNG5(bnProjectsPnc, adapter);
		}
		return singletonPNG5;
	}

    private String updateBnUserTable(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        BnUserAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnUserPKPNG _pngId =
                (jorm.BnUserPKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnUserPKPNG png2 =
                    (jorm.BnUserPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessor.paGetBnProperties(conn));
                jorm.BnUserPKPNG png3 =
                    (jorm.BnUserPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessor.paGetBnRoles(conn));
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    createdBnAgentsPnc.encodeAbstract(
                        paccessor.paGetCreatedBnAgents(conn));
                jorm.BnUserPKPNG png4 =
                    (jorm.BnUserPKPNG)
                    bnAuthRolesPnc.encodeAbstract(
                        paccessor.paGetBnAuthRoles(conn));
                jorm.BnUserPKPNG png5 =
                    (jorm.BnUserPKPNG)
                    bnProjectsPnc.encodeAbstract(
                        paccessor.paGetBnProjects(conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnUser SET password = ?, email = ?, name = ?, jabber = ?, id = ?, creationdate = ?, modificationdate = ? WHERE id = ?");
				//password
                nullField = paccessor.paGetPassword();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//email
                nullField = paccessor.paGetEmail();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//jabber
                nullField = paccessor.paGetJabber();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 6, ((java.util.Date) nullField), null);
                }
				//modificationdate
                nullField = paccessor.paGetModificationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 7, ((java.util.Date) nullField), null);
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                jorm.BnUserPKPNG png2 =
                    (jorm.BnUserPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProperties_NAME, conn));
                jorm.BnUserPKPNG png3 =
                    (jorm.BnUserPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnRoles_NAME, conn));
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    createdBnAgentsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(createdBnAgents_NAME, conn));
                jorm.BnUserPKPNG png4 =
                    (jorm.BnUserPKPNG)
                    bnAuthRolesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnAuthRoles_NAME, conn));
                jorm.BnUserPKPNG png5 =
                    (jorm.BnUserPKPNG)
                    bnProjectsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProjects_NAME, conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnUser SET password = ?, email = ?, name = ?, jabber = ?, id = ?, creationdate = ?, modificationdate = ? WHERE id = ?");
				//password
                nullField = paccessorGen.paGetStringField(password_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//email
                nullField = paccessorGen.paGetStringField(email_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessorGen.paGetStringField(name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//jabber
                nullField = paccessorGen.paGetStringField(jabber_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 6, ((java.util.Date) nullField), null);
                }
				//modificationdate
                nullField = paccessorGen.paGetDateField(modificationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 7, ((java.util.Date) nullField), null);
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertBnUserTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                BnUserAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnUserPKPNG _pngId =
                (jorm.BnUserPKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnUserPKPNG png2 =
                    (jorm.BnUserPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessor.paGetBnProperties(conn));
                jorm.BnUserPKPNG png3 =
                    (jorm.BnUserPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessor.paGetBnRoles(conn));
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    createdBnAgentsPnc.encodeAbstract(
                        paccessor.paGetCreatedBnAgents(conn));
                jorm.BnUserPKPNG png4 =
                    (jorm.BnUserPKPNG)
                    bnAuthRolesPnc.encodeAbstract(
                        paccessor.paGetBnAuthRoles(conn));
                jorm.BnUserPKPNG png5 =
                    (jorm.BnUserPKPNG)
                    bnProjectsPnc.encodeAbstract(
                        paccessor.paGetBnProjects(conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnUser (password, email, name, jabber, id, creationdate, modificationdate) VALUES (?, ?, ?, ?, ?, ?, ?)");
				//password
                nullField = paccessor.paGetPassword();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//email
                nullField = paccessor.paGetEmail();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//jabber
                nullField = paccessor.paGetJabber();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 6, ((java.util.Date) nullField), null);
                }
				//modificationdate
                nullField = paccessor.paGetModificationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 7, ((java.util.Date) nullField), null);
                }
            } else if (paccessorGen != null) {
                jorm.BnUserPKPNG png2 =
                    (jorm.BnUserPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProperties_NAME, conn));
                jorm.BnUserPKPNG png3 =
                    (jorm.BnUserPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnRoles_NAME, conn));
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    createdBnAgentsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(createdBnAgents_NAME, conn));
                jorm.BnUserPKPNG png4 =
                    (jorm.BnUserPKPNG)
                    bnAuthRolesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnAuthRoles_NAME, conn));
                jorm.BnUserPKPNG png5 =
                    (jorm.BnUserPKPNG)
                    bnProjectsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProjects_NAME, conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnUser (password, email, name, jabber, id, creationdate, modificationdate) VALUES (?, ?, ?, ?, ?, ?, ?)");
				//password
                nullField = paccessorGen.paGetStringField(password_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//email
                nullField = paccessorGen.paGetStringField(email_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessorGen.paGetStringField(name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//jabber
                nullField = paccessorGen.paGetStringField(jabber_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 6, ((java.util.Date) nullField), null);
                }
				//modificationdate
                nullField = paccessorGen.paGetDateField(modificationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 7, ((java.util.Date) nullField), null);
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteBnUserTable(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	BnUserAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM BnUser WHERE id = ?");
        jorm.BnUserPKPNG _pngId =
            (jorm.BnUserPKPNG)
            pname.encodeAbstract();
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("BnUser");
		JoinedTable.Join jtj = null;
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"BnUser.creationdate", "BnUser.email", "BnUser.id", "BnUser.jabber", "BnUser.modificationdate", "BnUser.name", "BnUser.password"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("id", "BnUser.id");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("BnUser.creationdate, BnUser.email, BnUser.id, BnUser.jabber, BnUser.modificationdate, BnUser.name, BnUser.password",
			Collections.singletonList(getJoinedTable()),
			"BnUser.id = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("BnUser.id",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("BnUser.creationdate, BnUser.email, BnUser.id, BnUser.jabber, BnUser.modificationdate, BnUser.name, BnUser.password, BnUser.id" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeAbstract(png, null);
	}



	
    // IMPLEMENTATION OF THE SPECIFIC PNAME GETTER
    public java.lang.String pnGetId(Object ctx)
            throws PException {
	    BnUserAccessor paccessor = null;
		if (!(ctx instanceof BnUserAccessor)) {
			throw new PExceptionProtocol("Unsupported context type: " + ctx);
		}
        try {
            return ((BnUserAccessor) ctx).paGetId();
        } catch (Exception e) {
		    throw new PException(e, "Cannot read the field 'id'");
        }
    }
    // IMPLEMENTATION OF THE PNameGetter INTERFACE

    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
		if ("id".equals(fn)) {
			if (ctx instanceof BnUserAccessor) {
				return ((BnUserAccessor) ctx).paGetId();
			} else {
			    throw new PExceptionProtocol("Unsupported context type: " + ctx);
			}
        }
        throw new PException("Cannot read the field " + fn);
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
		((Loggable) bnAuthRolesGCM).setLogger(l);
		((Loggable) bnProjectsGCM).setLogger(l);
		((Loggable) bnPropertiesGCM).setLogger(l);
		((Loggable) bnRolesGCM).setLogger(l);
		((Loggable) createdBnAgentsGCM).setLogger(l);
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}


/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnUserPNG2
    implements PNameGetter, jorm.BnUserPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnUserPNG2(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 3, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 3, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnUserPNG3
    implements PNameGetter, jorm.BnUserPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnUserPNG3(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 3, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 3, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnUserPNG1
    implements PNameGetter, jorm.BnUserPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnUserPNG1(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 3, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 3, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnUserPNG4
    implements PNameGetter, jorm.BnUserPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnUserPNG4(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 3, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 3, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnUserPNG5
    implements PNameGetter, jorm.BnUserPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnUserPNG5(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 3, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 3, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnUser_bnAuthRolesPNGPrefetch
        implements PNameGetter, jorm.BnUserPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnUser_bnAuthRolesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(3)) {
                    return ((Tuple) ctx).getString(3);
                } else {
                    return ((jorm.BnUserPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(3)) {
                        return ((Tuple) ctx).getString(3);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnUser_bnProjectsPNGPrefetch
        implements PNameGetter, jorm.BnUserPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnUser_bnProjectsPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(3)) {
                    return ((Tuple) ctx).getString(3);
                } else {
                    return ((jorm.BnUserPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(3)) {
                        return ((Tuple) ctx).getString(3);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnUser_bnPropertiesPNGPrefetch
        implements PNameGetter, jorm.BnUserPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnUser_bnPropertiesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(3)) {
                    return ((Tuple) ctx).getString(3);
                } else {
                    return ((jorm.BnUserPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(3)) {
                        return ((Tuple) ctx).getString(3);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnUser_bnRolesPNGPrefetch
        implements PNameGetter, jorm.BnUserPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnUser_bnRolesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(3)) {
                    return ((Tuple) ctx).getString(3);
                } else {
                    return ((jorm.BnUserPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(3)) {
                        return ((Tuple) ctx).getString(3);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnUser_createdBnAgentsPNGPrefetch
        implements PNameGetter, jorm.BnUserPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnUser_createdBnAgentsPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(3)) {
                    return ((Tuple) ctx).getString(3);
                } else {
                    return ((jorm.BnUserPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(3)) {
                        return ((Tuple) ctx).getString(3);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }



class BnUserResultIterator implements PNameIterator, jorm.BnUserPKPNG {
    ResultSet rs;
	private PBinder binder;
	private BnUserMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	BnUserResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		BnUserMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, -1, this);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }

    // IMPLEMENTATION OF THE BnUserPKPNG INTERFACE //

    public java.lang.String pnGetId(Object ctx) throws PExceptionIO, PExceptionTyping {
        try {
            return pclassMapping.adapter.getString(rs, pnameIndex + 0, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Impossible to fetch a the field id of the pname");
        }
    }
}
