/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.MedorEvaluator;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.genclass.RdbFieldDesc;
import org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassProp;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;


// Import clauses required by the object BnProject
import jorm.BnProjectAccessor;
import jorm.BnProjectPKPNG;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class BnProjectMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
	, PNameGetter, jorm.BnProjectPKPNG
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String bnAgentEdges_NAME = "bnAgentEdges";
	protected static final String bnAgents_NAME = "bnAgents";
	protected static final String bnEdges_NAME = "bnEdges";
	protected static final String bnHooks_NAME = "bnHooks";
	protected static final String bnInitiatorMapper_NAME = "bnInitiatorMapper";
	protected static final String bnInterHooks_NAME = "bnInterHooks";
	protected static final String bnIterations_NAME = "bnIterations";
	protected static final String bnNodes_NAME = "bnNodes";
	protected static final String bnProperties_NAME = "bnProperties";
	protected static final String bnRoles_NAME = "bnRoles";
	protected static final String bnUsers_NAME = "bnUsers";
	protected static final String creationDate_NAME = "creationDate";
	protected static final String creator_NAME = "creator";
	protected static final String endDate_NAME = "endDate";
	protected static final String id_NAME = "id";
	protected static final String name_NAME = "name";
	protected static final String parent_NAME = "parent";
	protected static final String state_NAME = "state";
	protected static final String status_NAME = "status";
	protected static final String type_NAME = "type";
	//Hidden fields
	protected static final String bnInitiatorMapper_id_NAME = "bnInitiatorMapper_id";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.BnProject";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder bnAgentEdgesPnc;
	PClassMapping bnAgentEdgesGCM;
	public PNameCoder bnAgentsPnc;
	PClassMapping bnAgentsGCM;
	public PNameCoder bnEdgesPnc;
	PClassMapping bnEdgesGCM;
	public PNameCoder bnHooksPnc;
	PClassMapping bnHooksGCM;
	public PNameCoder bnInitiatorMapperPnc;
	public PNameCoder bnInterHooksPnc;
	PClassMapping bnInterHooksGCM;
	public PNameCoder bnIterationsPnc;
	PClassMapping bnIterationsGCM;
	public PNameCoder bnNodesPnc;
	PClassMapping bnNodesGCM;
	public PNameCoder bnPropertiesPnc;
	PClassMapping bnPropertiesGCM;
	public PNameCoder bnRolesPnc;
	PClassMapping bnRolesGCM;
	public PNameCoder bnUsersPnc;
	PClassMapping bnUsersGCM;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public BnProjectMapping() {
 	}
	public PNameCoder getBnAgentEdgesPNameCoder() {
	    return bnAgentEdgesPnc;
	}
	public PNameCoder getBnAgentsPNameCoder() {
	    return bnAgentsPnc;
	}
	public PNameCoder getBnEdgesPNameCoder() {
	    return bnEdgesPnc;
	}
	public PNameCoder getBnHooksPNameCoder() {
	    return bnHooksPnc;
	}
	public PNameCoder getBnInitiatorMapperPNameCoder() {
	    return bnInitiatorMapperPnc;
	}
	public PNameCoder getBnInterHooksPNameCoder() {
	    return bnInterHooksPnc;
	}
	public PNameCoder getBnIterationsPNameCoder() {
	    return bnIterationsPnc;
	}
	public PNameCoder getBnNodesPNameCoder() {
	    return bnNodesPnc;
	}
	public PNameCoder getBnPropertiesPNameCoder() {
	    return bnPropertiesPnc;
	}
	public PNameCoder getBnRolesPNameCoder() {
	    return bnRolesPnc;
	}
	public PNameCoder getBnUsersPNameCoder() {
	    return bnUsersPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (bnAgentEdges_NAME.equals(fn)) {
			bnAgentEdgesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnAgents_NAME.equals(fn)) {
			bnAgentsPnc = (PNamingContext) pnc;
			return;
		}
		if (bnEdges_NAME.equals(fn)) {
			bnEdgesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnHooks_NAME.equals(fn)) {
			bnHooksPnc = (PNamingContext) pnc;
			return;
		}
		if (bnInitiatorMapper_NAME.equals(fn)) {
			bnInitiatorMapperPnc = (PNamingContext) pnc;
			return;
		}
		if (bnInterHooks_NAME.equals(fn)) {
			bnInterHooksPnc = (PNamingContext) pnc;
			return;
		}
		if (bnIterations_NAME.equals(fn)) {
			bnIterationsPnc = (PNamingContext) pnc;
			return;
		}
		if (bnNodes_NAME.equals(fn)) {
			bnNodesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnProperties_NAME.equals(fn)) {
			bnPropertiesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnRoles_NAME.equals(fn)) {
			bnRolesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnUsers_NAME.equals(fn)) {
			bnUsersPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		if (bnAgentEdges_NAME.equals(fn)) {
			bnAgentEdgesGCM = pcm;
			return;
		}
		if (bnAgents_NAME.equals(fn)) {
			bnAgentsGCM = pcm;
			return;
		}
		if (bnEdges_NAME.equals(fn)) {
			bnEdgesGCM = pcm;
			return;
		}
		if (bnHooks_NAME.equals(fn)) {
			bnHooksGCM = pcm;
			return;
		}
		if (bnInterHooks_NAME.equals(fn)) {
			bnInterHooksGCM = pcm;
			return;
		}
		if (bnIterations_NAME.equals(fn)) {
			bnIterationsGCM = pcm;
			return;
		}
		if (bnNodes_NAME.equals(fn)) {
			bnNodesGCM = pcm;
			return;
		}
		if (bnProperties_NAME.equals(fn)) {
			bnPropertiesGCM = pcm;
			return;
		}
		if (bnRoles_NAME.equals(fn)) {
			bnRolesGCM = pcm;
			return;
		}
		if (bnUsers_NAME.equals(fn)) {
			bnUsersGCM = pcm;
			return;
		}
		throw new UnsupportedOperationException("This class does not contain a " + fn + " field referencing a generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.BnProject",
				SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference bnInitiatorMapper
        String[][] bnInitiatorMapperICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BnInitiatorMapper",
            bnInitiatorMapperICDef);
        bnInitiatorMapperPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}

         // Initialization of the gen class /jorm.BnProject/bnRoles
        FieldDesc[] fds = null;
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnRolesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnRolesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnRole_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnRolesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnRolesGCM).setTableName("BnRole");
			((RdbGenClassProp) bnRolesGCM).setColocated(true);
			((RdbGenClassProp) bnRolesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnRolesGCM).setReadOnly(true);
			((RdbGenClassProp) bnRolesGCM).setTypeConverter(adapter);
        String[] bnRolesGCDef = {"java.util.Collection"};
        String[][] bnRolesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnRolesGCDef,
            "jorm.BnRole",
            bnRolesICDef);
        ((GenClassProp) bnRolesGCM).setPType(ptype);
        bnRolesPnc.setPType(ptype);
        bnRolesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnAgentEdges
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnAgentEdgesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnAgentEdgesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnAgentEdge_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnAgentEdgesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnAgentEdgesGCM).setTableName("BnAgentEdge");
			((RdbGenClassProp) bnAgentEdgesGCM).setColocated(true);
			((RdbGenClassProp) bnAgentEdgesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnAgentEdgesGCM).setReadOnly(true);
			((RdbGenClassProp) bnAgentEdgesGCM).setTypeConverter(adapter);
        String[] bnAgentEdgesGCDef = {"java.util.Collection"};
        String[][] bnAgentEdgesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnAgentEdgesGCDef,
            "jorm.BnAgentEdge",
            bnAgentEdgesICDef);
        ((GenClassProp) bnAgentEdgesGCM).setPType(ptype);
        bnAgentEdgesPnc.setPType(ptype);
        bnAgentEdgesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnEdges
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnEdgesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnEdgesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnEdge_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnEdgesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnEdgesGCM).setTableName("BnEdge");
			((RdbGenClassProp) bnEdgesGCM).setColocated(true);
			((RdbGenClassProp) bnEdgesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnEdgesGCM).setReadOnly(true);
			((RdbGenClassProp) bnEdgesGCM).setTypeConverter(adapter);
        String[] bnEdgesGCDef = {"java.util.Collection"};
        String[][] bnEdgesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnEdgesGCDef,
            "jorm.BnEdge",
            bnEdgesICDef);
        ((GenClassProp) bnEdgesGCM).setPType(ptype);
        bnEdgesPnc.setPType(ptype);
        bnEdgesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnNodes
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnNodesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnNodesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnNodesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnNodesGCM).setTableName("BnNode");
			((RdbGenClassProp) bnNodesGCM).setColocated(true);
			((RdbGenClassProp) bnNodesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnNodesGCM).setReadOnly(true);
			((RdbGenClassProp) bnNodesGCM).setTypeConverter(adapter);
        String[] bnNodesGCDef = {"java.util.Collection"};
        String[][] bnNodesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnNodesGCDef,
            "jorm.BnNode",
            bnNodesICDef);
        ((GenClassProp) bnNodesGCM).setPType(ptype);
        bnNodesPnc.setPType(ptype);
        bnNodesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnAgents
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnAgentsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnAgentsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnAgent_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnAgentsGCM).setElemDesc(fds);
			((RdbGenClassProp) bnAgentsGCM).setTableName("BnAgent");
			((RdbGenClassProp) bnAgentsGCM).setColocated(true);
			((RdbGenClassProp) bnAgentsGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnAgentsGCM).setReadOnly(true);
			((RdbGenClassProp) bnAgentsGCM).setTypeConverter(adapter);
        String[] bnAgentsGCDef = {"java.util.Collection"};
        String[][] bnAgentsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnAgentsGCDef,
            "jorm.BnAgent",
            bnAgentsICDef);
        ((GenClassProp) bnAgentsGCM).setPType(ptype);
        bnAgentsPnc.setPType(ptype);
        bnAgentsGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnHooks
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnHooksGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnHooksGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnProjectHook_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnHooksGCM).setElemDesc(fds);
			((RdbGenClassProp) bnHooksGCM).setTableName("BnProjectHook");
			((RdbGenClassProp) bnHooksGCM).setColocated(true);
			((RdbGenClassProp) bnHooksGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnHooksGCM).setReadOnly(true);
			((RdbGenClassProp) bnHooksGCM).setTypeConverter(adapter);
        String[] bnHooksGCDef = {"java.util.Collection"};
        String[][] bnHooksICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnHooksGCDef,
            "jorm.BnProjectHook",
            bnHooksICDef);
        ((GenClassProp) bnHooksGCM).setPType(ptype);
        bnHooksPnc.setPType(ptype);
        bnHooksGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnIterations
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnIterationsGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnIterationsGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnIteration_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnIterationsGCM).setElemDesc(fds);
			((RdbGenClassProp) bnIterationsGCM).setTableName("BnIteration");
			((RdbGenClassProp) bnIterationsGCM).setColocated(true);
			((RdbGenClassProp) bnIterationsGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnIterationsGCM).setReadOnly(true);
			((RdbGenClassProp) bnIterationsGCM).setTypeConverter(adapter);
        String[] bnIterationsGCDef = {"java.util.Collection"};
        String[][] bnIterationsICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnIterationsGCDef,
            "jorm.BnIteration",
            bnIterationsICDef);
        ((GenClassProp) bnIterationsGCM).setPType(ptype);
        bnIterationsPnc.setPType(ptype);
        bnIterationsGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnInterHooks
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnInterHooksGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnInterHooksGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnProjectInterHook_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnInterHooksGCM).setElemDesc(fds);
			((RdbGenClassProp) bnInterHooksGCM).setTableName("BnProjectInterHook");
			((RdbGenClassProp) bnInterHooksGCM).setColocated(true);
			((RdbGenClassProp) bnInterHooksGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnInterHooksGCM).setReadOnly(true);
			((RdbGenClassProp) bnInterHooksGCM).setTypeConverter(adapter);
        String[] bnInterHooksGCDef = {"java.util.Collection"};
        String[][] bnInterHooksICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnInterHooksGCDef,
            "jorm.BnProjectInterHook",
            bnInterHooksICDef);
        ((GenClassProp) bnInterHooksGCM).setPType(ptype);
        bnInterHooksPnc.setPType(ptype);
        bnInterHooksGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnUsers
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnUsersGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnUsersGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnUser_id", "id", PTypeSpace.STRING, "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnUsersGCM).setElemDesc(fds);
			((RdbGenClassProp) bnUsersGCM).setTableName("BNUSER_BNPROJECT");
			((RdbGenClassProp) bnUsersGCM).setColocated(false);
			((RdbGenClassProp) bnUsersGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnUsersGCM).setReadOnly(false);
			((RdbGenClassProp) bnUsersGCM).setTypeConverter(adapter);
        String[] bnUsersGCDef = {"java.util.Collection"};
        String[][] bnUsersICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnUsersGCDef,
            "jorm.BnUser",
            bnUsersICDef);
        ((GenClassProp) bnUsersGCM).setPType(ptype);
        bnUsersPnc.setPType(ptype);
        bnUsersGCM.init(m, null);

         // Initialization of the gen class /jorm.BnProject/bnProperties
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnProject_id", "id", PTypeSpace.STRING, "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnPropertiesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnPropertiesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnProjectProperty_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnPropertiesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnPropertiesGCM).setTableName("BnProjectProperty");
			((RdbGenClassProp) bnPropertiesGCM).setColocated(true);
			((RdbGenClassProp) bnPropertiesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnPropertiesGCM).setReadOnly(true);
			((RdbGenClassProp) bnPropertiesGCM).setTypeConverter(adapter);
        String[] bnPropertiesGCDef = {"java.util.Collection"};
        String[][] bnPropertiesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnPropertiesGCDef,
            "jorm.BnProjectProperty",
            bnPropertiesICDef);
        ((GenClassProp) bnPropertiesGCM).setPType(ptype);
        bnPropertiesPnc.setPType(ptype);
        bnPropertiesGCM.init(m, null);
 		//register the structures and dependencies
		init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.BnProject
			        	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "creator", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "enddate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "type", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "BnInitiatorMapper_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "state", adapter.getSqlType(PTypeSpace.INT.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "status", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "name", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, -1, -1), true, true, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "parent", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnProject", "creationdate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	    		        // Initialization of the structures of the gen class /jorm.BnProject/bnRoles
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnRole", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnRole", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnAgentEdges
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgentEdge", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgentEdge", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnEdges
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnEdge", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnEdge", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnNodes
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnAgents
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgent", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgent", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnHooks
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnProjectHook", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnProjectHook", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnIterations
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnIteration", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnIteration", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnInterHooks
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnProjectInterHook", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnProjectInterHook", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnUsers
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNUSER_BNPROJECT", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BNUSER_BNPROJECT", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnProject/bnProperties
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnProjectProperty", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnProjectProperty", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
						//dependencies
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnAgentEdge");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnInitiatorMapper");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnNode");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnProjectProperty");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnProjectHook");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnUser");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnProjectInterHook");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnEdge");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnRole");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnAgent");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnIteration");
		} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (bnAgentEdges_NAME.equals(fn))
				return bnAgentEdgesPnc;
			if (bnAgents_NAME.equals(fn))
				return bnAgentsPnc;
			if (bnEdges_NAME.equals(fn))
				return bnEdgesPnc;
			if (bnHooks_NAME.equals(fn))
				return bnHooksPnc;
			if (bnInitiatorMapper_NAME.equals(fn))
				return bnInitiatorMapperPnc;
			if (bnInterHooks_NAME.equals(fn))
				return bnInterHooksPnc;
			if (bnIterations_NAME.equals(fn))
				return bnIterationsPnc;
			if (bnNodes_NAME.equals(fn))
				return bnNodesPnc;
			if (bnProperties_NAME.equals(fn))
				return bnPropertiesPnc;
			if (bnRoles_NAME.equals(fn))
				return bnRolesPnc;
			if (bnUsers_NAME.equals(fn))
				return bnUsersPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			if (bnAgentEdges_NAME.equals(fn))
				return bnAgentEdgesGCM;
			if (bnAgents_NAME.equals(fn))
				return bnAgentsGCM;
			if (bnEdges_NAME.equals(fn))
				return bnEdgesGCM;
			if (bnHooks_NAME.equals(fn))
				return bnHooksGCM;
			if (bnInterHooks_NAME.equals(fn))
				return bnInterHooksGCM;
			if (bnIterations_NAME.equals(fn))
				return bnIterationsGCM;
			if (bnNodes_NAME.equals(fn))
				return bnNodesGCM;
			if (bnProperties_NAME.equals(fn))
				return bnPropertiesGCM;
			if (bnRoles_NAME.equals(fn))
				return bnRolesGCM;
			if (bnUsers_NAME.equals(fn))
				return bnUsersGCM;
			throw new UnsupportedOperationException("This class does not contain a " + fn + " field which would reference a generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.BnProjectBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
			if(withSubType){
				//the list of fields belonging to the primary key
				String pk = "BnProject.id";
				queryToExecute = rdbcm.getRdbInheritanceQuery().getExtentQuery(mapper,prefetching, true, pk, this);	
				//System.out.println("query to execute:" + queryToExecute + " compared with the generated query: " + (prefetching ? prefetchedExtentQuery : extentQuery));
				beginIndex =  rdbcm.getRdbInheritanceQuery().getExtentFieldsNames(prefetching, true).size() + 1;
			}
			else{
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 10 + 1;
			}
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new BnProjectResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <BnProject>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <BnProject>.");
        }
		return it;
	}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
					return false;
				}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeAbstract(pname.encodeAbstract(), null);
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
        PClassMapping gcm = null;
        bnAgentEdgesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnAgentEdges_NAME, new String[]{"java.util.Collection"});
        bnAgentEdgesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnAgentEdges_NAME, new String[]{"java.util.Collection"}, "jorm.BnAgentEdge");
        gcm = bnAgentEdgesGCM;
        bnAgentsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnAgents_NAME, new String[]{"java.util.Collection"});
        bnAgentsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnAgents_NAME, new String[]{"java.util.Collection"}, "jorm.BnAgent");
        gcm = bnAgentsGCM;
        bnEdgesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnEdges_NAME, new String[]{"java.util.Collection"});
        bnEdgesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnEdges_NAME, new String[]{"java.util.Collection"}, "jorm.BnEdge");
        gcm = bnEdgesGCM;
        bnHooksPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnHooks_NAME, new String[]{"java.util.Collection"});
        bnHooksGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnHooks_NAME, new String[]{"java.util.Collection"}, "jorm.BnProjectHook");
        gcm = bnHooksGCM;
		bnInitiatorMapperPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnInitiatorMapper_NAME, "jorm.BnInitiatorMapper");
        bnInterHooksPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnInterHooks_NAME, new String[]{"java.util.Collection"});
        bnInterHooksGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnInterHooks_NAME, new String[]{"java.util.Collection"}, "jorm.BnProjectInterHook");
        gcm = bnInterHooksGCM;
        bnIterationsPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnIterations_NAME, new String[]{"java.util.Collection"});
        bnIterationsGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnIterations_NAME, new String[]{"java.util.Collection"}, "jorm.BnIteration");
        gcm = bnIterationsGCM;
        bnNodesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnNodes_NAME, new String[]{"java.util.Collection"});
        bnNodesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnNodes_NAME, new String[]{"java.util.Collection"}, "jorm.BnNode");
        gcm = bnNodesGCM;
        bnPropertiesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnProperties_NAME, new String[]{"java.util.Collection"});
        bnPropertiesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnProperties_NAME, new String[]{"java.util.Collection"}, "jorm.BnProjectProperty");
        gcm = bnPropertiesGCM;
        bnRolesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnRoles_NAME, new String[]{"java.util.Collection"});
        bnRolesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnRoles_NAME, new String[]{"java.util.Collection"}, "jorm.BnRole");
        gcm = bnRolesGCM;
        bnUsersPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnUsers_NAME, new String[]{"java.util.Collection"});
        bnUsersGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnUsers_NAME, new String[]{"java.util.Collection"}, "jorm.BnUser");
        gcm = bnUsersGCM;
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		BnProjectAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnProjectAccessor)
			paccessor = (BnProjectAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table BnProject : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteBnProjectTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnProject]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertBnProjectTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnProject]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateBnProjectTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnProject]: problematic SQL query -> {" + sqlquery + "}");
                }
		}
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}

	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		BnProjectAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnProjectAccessor)
			paccessor = (BnProjectAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            pstmt = connection.prepareStatement(readQuery);
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnProjectPKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnProjectPKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false)
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " + pname);
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetCreationDate(adapter.getDate(resultSet, 2, null, null));
                paccessor.paSetCreator(adapter.getString(resultSet, 3, null));
                paccessor.paSetEndDate(adapter.getDate(resultSet, 4, null, null));
                paccessor.paSetId(adapter.getString(resultSet, 5, null));
                paccessor.paSetName(adapter.getString(resultSet, 6, null));
                paccessor.paSetParent(adapter.getString(resultSet, 7, null));
                paccessor.paSetState(adapter.getInt(resultSet, 8, ((int) -1)));
                paccessor.paSetStatus(adapter.getString(resultSet, 9, null));
                paccessor.paSetType(adapter.getString(resultSet, 10, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@18a62f6 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@5a936b / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnInterHooks / join=null
                refname = bnInterHooksPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessor.paSetBnInterHooks(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1539d49 / cnId=4 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1f13b08 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnEdges / join=null
                refname = bnEdgesPnc.decodeAbstract(getInstancePNG4(), resultSet);
                paccessor.paSetBnEdges(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@7b5cb8 / cnId=10 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@d8a923 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnProperties / join=null
                refname = bnPropertiesPnc.decodeAbstract(getInstancePNG10(), resultSet);
                paccessor.paSetBnProperties(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1046270 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@16a5d72 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnRoles / join=null
                refname = bnRolesPnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessor.paSetBnRoles(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@dad4b8 / cnId=7 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@16b152b / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnIterations / join=null
                refname = bnIterationsPnc.decodeAbstract(getInstancePNG7(), resultSet);
                paccessor.paSetBnIterations(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@c39410 / cnId=1 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@f6af3b / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnAgentEdges / join=null
                refname = bnAgentEdgesPnc.decodeAbstract(getInstancePNG1(), resultSet);
                paccessor.paSetBnAgentEdges(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@2db73e / cnId=9 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1c9d17e / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnHooks / join=null
                refname = bnHooksPnc.decodeAbstract(getInstancePNG9(), resultSet);
                paccessor.paSetBnHooks(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1594ba3 / cnId=6 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@84917a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnNodes / join=null
                refname = bnNodesPnc.decodeAbstract(getInstancePNG6(), resultSet);
                paccessor.paSetBnNodes(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@174f478 / cnId=8 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@15e0163} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@13abee1 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@14a7a12 / nd.nr.cn={id=bnInitiatorMapper_id} / fieldName=bnInitiatorMapper / join=null
                refname = bnInitiatorMapperPnc.decodeAbstract(getInstancePNG8(), resultSet);
                paccessor.paSetBnInitiatorMapper(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@ad40a0 / cnId=3 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1d07e4 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnAgents / join=null
                refname = bnAgentsPnc.decodeAbstract(getInstancePNG3(), resultSet);
                paccessor.paSetBnAgents(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessor.paSetBnUsers(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetDateField(creationDate_NAME, adapter.getDate(resultSet, 2, null, null));
                paccessorGen.paSetStringField(creator_NAME, adapter.getString(resultSet, 3, null));
                paccessorGen.paSetDateField(endDate_NAME, adapter.getDate(resultSet, 4, null, null));
                paccessorGen.paSetStringField(id_NAME, adapter.getString(resultSet, 5, null));
                paccessorGen.paSetStringField(name_NAME, adapter.getString(resultSet, 6, null));
                paccessorGen.paSetStringField(parent_NAME, adapter.getString(resultSet, 7, null));
                paccessorGen.paSetIntField(state_NAME, adapter.getInt(resultSet, 8, ((int) -1)));
                paccessorGen.paSetStringField(status_NAME, adapter.getString(resultSet, 9, null));
                paccessorGen.paSetStringField(type_NAME, adapter.getString(resultSet, 10, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@6dd60e / cnId=11 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@6742d0} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4854f6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@435a3a / nd.nr.cn={id=id} / fieldName=bnUsers / join=null
                refname = bnUsersPnc.decodeAbstract(getInstancePNG11(), resultSet);
                paccessorGen.paSetRefField(bnUsers_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa);
	            return;
	        }
		    BnProjectAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof BnProjectAccessor)
			    paccessor = (BnProjectAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
                paccessor.paSetBnAgentEdges(bnAgentEdgesPnc.decodeAbstract(getInstancePNGPrefetchbnAgentEdges(), tuple), conn);
                paccessor.paSetBnAgents(bnAgentsPnc.decodeAbstract(getInstancePNGPrefetchbnAgents(), tuple), conn);
                paccessor.paSetBnEdges(bnEdgesPnc.decodeAbstract(getInstancePNGPrefetchbnEdges(), tuple), conn);
                paccessor.paSetBnHooks(bnHooksPnc.decodeAbstract(getInstancePNGPrefetchbnHooks(), tuple), conn);
                paccessor.paSetBnInitiatorMapper(bnInitiatorMapperPnc.decodeAbstract(getInstancePNGPrefetchbnInitiatorMapper(), tuple), conn);
                paccessor.paSetBnInterHooks(bnInterHooksPnc.decodeAbstract(getInstancePNGPrefetchbnInterHooks(), tuple), conn);
                paccessor.paSetBnIterations(bnIterationsPnc.decodeAbstract(getInstancePNGPrefetchbnIterations(), tuple), conn);
                paccessor.paSetBnNodes(bnNodesPnc.decodeAbstract(getInstancePNGPrefetchbnNodes(), tuple), conn);
                paccessor.paSetBnProperties(bnPropertiesPnc.decodeAbstract(getInstancePNGPrefetchbnProperties(), tuple), conn);
                paccessor.paSetBnRoles(bnRolesPnc.decodeAbstract(getInstancePNGPrefetchbnRoles(), tuple), conn);
                paccessor.paSetBnUsers(bnUsersPnc.decodeAbstract(getInstancePNGPrefetchbnUsers(), tuple), conn);
                paccessor.paSetCreationDate(tuple.getDate(2));
                paccessor.paSetCreator(tuple.getString(3));
                paccessor.paSetEndDate(tuple.getDate(4));
                paccessor.paSetId(tuple.getString(5));
                paccessor.paSetName(tuple.getString(6));
                paccessor.paSetParent(tuple.getString(7));
                paccessor.paSetState(tuple.getInt(8));
                paccessor.paSetStatus(tuple.getString(9));
                paccessor.paSetType(tuple.getString(10));
            } else if (paccessorGen != null) {
                paccessorGen.paSetRefField(bnAgentEdges_NAME, bnAgentEdgesPnc.decodeAbstract(getInstancePNGPrefetchbnAgentEdges(), tuple), conn);
                paccessorGen.paSetRefField(bnAgents_NAME, bnAgentsPnc.decodeAbstract(getInstancePNGPrefetchbnAgents(), tuple), conn);
                paccessorGen.paSetRefField(bnEdges_NAME, bnEdgesPnc.decodeAbstract(getInstancePNGPrefetchbnEdges(), tuple), conn);
                paccessorGen.paSetRefField(bnHooks_NAME, bnHooksPnc.decodeAbstract(getInstancePNGPrefetchbnHooks(), tuple), conn);
                paccessorGen.paSetRefField(bnInitiatorMapper_NAME, bnInitiatorMapperPnc.decodeAbstract(getInstancePNGPrefetchbnInitiatorMapper(), tuple), conn);
                paccessorGen.paSetRefField(bnInterHooks_NAME, bnInterHooksPnc.decodeAbstract(getInstancePNGPrefetchbnInterHooks(), tuple), conn);
                paccessorGen.paSetRefField(bnIterations_NAME, bnIterationsPnc.decodeAbstract(getInstancePNGPrefetchbnIterations(), tuple), conn);
                paccessorGen.paSetRefField(bnNodes_NAME, bnNodesPnc.decodeAbstract(getInstancePNGPrefetchbnNodes(), tuple), conn);
                paccessorGen.paSetRefField(bnProperties_NAME, bnPropertiesPnc.decodeAbstract(getInstancePNGPrefetchbnProperties(), tuple), conn);
                paccessorGen.paSetRefField(bnRoles_NAME, bnRolesPnc.decodeAbstract(getInstancePNGPrefetchbnRoles(), tuple), conn);
                paccessorGen.paSetRefField(bnUsers_NAME, bnUsersPnc.decodeAbstract(getInstancePNGPrefetchbnUsers(), tuple), conn);
                paccessorGen.paSetDateField(creationDate_NAME, tuple.getDate(2));
                paccessorGen.paSetStringField(creator_NAME, tuple.getString(3));
                paccessorGen.paSetDateField(endDate_NAME, tuple.getDate(4));
                paccessorGen.paSetStringField(id_NAME, tuple.getString(5));
                paccessorGen.paSetStringField(name_NAME, tuple.getString(6));
                paccessorGen.paSetStringField(parent_NAME, tuple.getString(7));
                paccessorGen.paSetIntField(state_NAME, tuple.getInt(8));
                paccessorGen.paSetStringField(status_NAME, tuple.getString(9));
                paccessorGen.paSetStringField(type_NAME, tuple.getString(10));
            }
			((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT BnProject.id FROM BnProject WHERE BnProject.id = ?");
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnProjectPKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnProjectPKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}
    private static BnProject_bnAgentEdgesPNGPrefetch singletonPNGPrefetchbnAgentEdges = null;

    private BnProject_bnAgentEdgesPNGPrefetch getInstancePNGPrefetchbnAgentEdges() throws PException {
        if (singletonPNGPrefetchbnAgentEdges==null) {
            singletonPNGPrefetchbnAgentEdges = new BnProject_bnAgentEdgesPNGPrefetch(bnAgentEdgesPnc);
		}
		return singletonPNGPrefetchbnAgentEdges;
	}
    private static BnProject_bnAgentsPNGPrefetch singletonPNGPrefetchbnAgents = null;

    private BnProject_bnAgentsPNGPrefetch getInstancePNGPrefetchbnAgents() throws PException {
        if (singletonPNGPrefetchbnAgents==null) {
            singletonPNGPrefetchbnAgents = new BnProject_bnAgentsPNGPrefetch(bnAgentsPnc);
		}
		return singletonPNGPrefetchbnAgents;
	}
    private static BnProject_bnEdgesPNGPrefetch singletonPNGPrefetchbnEdges = null;

    private BnProject_bnEdgesPNGPrefetch getInstancePNGPrefetchbnEdges() throws PException {
        if (singletonPNGPrefetchbnEdges==null) {
            singletonPNGPrefetchbnEdges = new BnProject_bnEdgesPNGPrefetch(bnEdgesPnc);
		}
		return singletonPNGPrefetchbnEdges;
	}
    private static BnProject_bnHooksPNGPrefetch singletonPNGPrefetchbnHooks = null;

    private BnProject_bnHooksPNGPrefetch getInstancePNGPrefetchbnHooks() throws PException {
        if (singletonPNGPrefetchbnHooks==null) {
            singletonPNGPrefetchbnHooks = new BnProject_bnHooksPNGPrefetch(bnHooksPnc);
		}
		return singletonPNGPrefetchbnHooks;
	}
    private static BnProject_bnInitiatorMapperPNGPrefetch singletonPNGPrefetchbnInitiatorMapper = null;

    private BnProject_bnInitiatorMapperPNGPrefetch getInstancePNGPrefetchbnInitiatorMapper() throws PException {
        if (singletonPNGPrefetchbnInitiatorMapper==null) {
            singletonPNGPrefetchbnInitiatorMapper = new BnProject_bnInitiatorMapperPNGPrefetch(bnInitiatorMapperPnc);
		}
		return singletonPNGPrefetchbnInitiatorMapper;
	}
    private static BnProject_bnInterHooksPNGPrefetch singletonPNGPrefetchbnInterHooks = null;

    private BnProject_bnInterHooksPNGPrefetch getInstancePNGPrefetchbnInterHooks() throws PException {
        if (singletonPNGPrefetchbnInterHooks==null) {
            singletonPNGPrefetchbnInterHooks = new BnProject_bnInterHooksPNGPrefetch(bnInterHooksPnc);
		}
		return singletonPNGPrefetchbnInterHooks;
	}
    private static BnProject_bnIterationsPNGPrefetch singletonPNGPrefetchbnIterations = null;

    private BnProject_bnIterationsPNGPrefetch getInstancePNGPrefetchbnIterations() throws PException {
        if (singletonPNGPrefetchbnIterations==null) {
            singletonPNGPrefetchbnIterations = new BnProject_bnIterationsPNGPrefetch(bnIterationsPnc);
		}
		return singletonPNGPrefetchbnIterations;
	}
    private static BnProject_bnNodesPNGPrefetch singletonPNGPrefetchbnNodes = null;

    private BnProject_bnNodesPNGPrefetch getInstancePNGPrefetchbnNodes() throws PException {
        if (singletonPNGPrefetchbnNodes==null) {
            singletonPNGPrefetchbnNodes = new BnProject_bnNodesPNGPrefetch(bnNodesPnc);
		}
		return singletonPNGPrefetchbnNodes;
	}
    private static BnProject_bnPropertiesPNGPrefetch singletonPNGPrefetchbnProperties = null;

    private BnProject_bnPropertiesPNGPrefetch getInstancePNGPrefetchbnProperties() throws PException {
        if (singletonPNGPrefetchbnProperties==null) {
            singletonPNGPrefetchbnProperties = new BnProject_bnPropertiesPNGPrefetch(bnPropertiesPnc);
		}
		return singletonPNGPrefetchbnProperties;
	}
    private static BnProject_bnRolesPNGPrefetch singletonPNGPrefetchbnRoles = null;

    private BnProject_bnRolesPNGPrefetch getInstancePNGPrefetchbnRoles() throws PException {
        if (singletonPNGPrefetchbnRoles==null) {
            singletonPNGPrefetchbnRoles = new BnProject_bnRolesPNGPrefetch(bnRolesPnc);
		}
		return singletonPNGPrefetchbnRoles;
	}
    private static BnProject_bnUsersPNGPrefetch singletonPNGPrefetchbnUsers = null;

    private BnProject_bnUsersPNGPrefetch getInstancePNGPrefetchbnUsers() throws PException {
        if (singletonPNGPrefetchbnUsers==null) {
            singletonPNGPrefetchbnUsers = new BnProject_bnUsersPNGPrefetch(bnUsersPnc);
		}
		return singletonPNGPrefetchbnUsers;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private static jorm.BnProjectPKPNG singletonPNG5 = null;
    public jorm.BnProjectPKPNG getInstancePNG5() throws PException {
        if (singletonPNG5==null) {
            singletonPNG5 = new  BnProjectPNG5(bnInterHooksPnc, adapter);
		}
		return singletonPNG5;
	}

    private static jorm.BnProjectPKPNG singletonPNG4 = null;
    public jorm.BnProjectPKPNG getInstancePNG4() throws PException {
        if (singletonPNG4==null) {
            singletonPNG4 = new  BnProjectPNG4(bnEdgesPnc, adapter);
		}
		return singletonPNG4;
	}

    private static jorm.BnProjectPKPNG singletonPNG10 = null;
    public jorm.BnProjectPKPNG getInstancePNG10() throws PException {
        if (singletonPNG10==null) {
            singletonPNG10 = new  BnProjectPNG10(bnPropertiesPnc, adapter);
		}
		return singletonPNG10;
	}

    private static jorm.BnProjectPKPNG singletonPNG2 = null;
    public jorm.BnProjectPKPNG getInstancePNG2() throws PException {
        if (singletonPNG2==null) {
            singletonPNG2 = new  BnProjectPNG2(bnRolesPnc, adapter);
		}
		return singletonPNG2;
	}

    private static jorm.BnProjectPKPNG singletonPNG7 = null;
    public jorm.BnProjectPKPNG getInstancePNG7() throws PException {
        if (singletonPNG7==null) {
            singletonPNG7 = new  BnProjectPNG7(bnIterationsPnc, adapter);
		}
		return singletonPNG7;
	}

    private static jorm.BnProjectPKPNG singletonPNG1 = null;
    public jorm.BnProjectPKPNG getInstancePNG1() throws PException {
        if (singletonPNG1==null) {
            singletonPNG1 = new  BnProjectPNG1(bnAgentEdgesPnc, adapter);
		}
		return singletonPNG1;
	}

    private static jorm.BnProjectPKPNG singletonPNG9 = null;
    public jorm.BnProjectPKPNG getInstancePNG9() throws PException {
        if (singletonPNG9==null) {
            singletonPNG9 = new  BnProjectPNG9(bnHooksPnc, adapter);
		}
		return singletonPNG9;
	}

    private static jorm.BnProjectPKPNG singletonPNG6 = null;
    public jorm.BnProjectPKPNG getInstancePNG6() throws PException {
        if (singletonPNG6==null) {
            singletonPNG6 = new  BnProjectPNG6(bnNodesPnc, adapter);
		}
		return singletonPNG6;
	}

    private static jorm.BnInitiatorMapperPKPNG singletonPNG8 = null;
    public jorm.BnInitiatorMapperPKPNG getInstancePNG8() throws PException {
        if (singletonPNG8==null) {
            singletonPNG8 = new  BnProjectPNG8(bnInitiatorMapperPnc, adapter);
		}
		return singletonPNG8;
	}

    private static jorm.BnProjectPKPNG singletonPNG3 = null;
    public jorm.BnProjectPKPNG getInstancePNG3() throws PException {
        if (singletonPNG3==null) {
            singletonPNG3 = new  BnProjectPNG3(bnAgentsPnc, adapter);
		}
		return singletonPNG3;
	}

    private static jorm.BnProjectPKPNG singletonPNG11 = null;
    public jorm.BnProjectPKPNG getInstancePNG11() throws PException {
        if (singletonPNG11==null) {
            singletonPNG11 = new  BnProjectPNG11(bnUsersPnc, adapter);
		}
		return singletonPNG11;
	}

    private String updateBnProjectTable(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        BnProjectAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnProjectPKPNG _pngId =
                (jorm.BnProjectPKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessor.paGetBnInterHooks(conn));
                jorm.BnProjectPKPNG png4 =
                    (jorm.BnProjectPKPNG)
                    bnEdgesPnc.encodeAbstract(
                        paccessor.paGetBnEdges(conn));
                jorm.BnProjectPKPNG png10 =
                    (jorm.BnProjectPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessor.paGetBnProperties(conn));
                jorm.BnProjectPKPNG png2 =
                    (jorm.BnProjectPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessor.paGetBnRoles(conn));
                jorm.BnProjectPKPNG png7 =
                    (jorm.BnProjectPKPNG)
                    bnIterationsPnc.encodeAbstract(
                        paccessor.paGetBnIterations(conn));
                jorm.BnProjectPKPNG png1 =
                    (jorm.BnProjectPKPNG)
                    bnAgentEdgesPnc.encodeAbstract(
                        paccessor.paGetBnAgentEdges(conn));
                jorm.BnProjectPKPNG png9 =
                    (jorm.BnProjectPKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessor.paGetBnHooks(conn));
                jorm.BnProjectPKPNG png6 =
                    (jorm.BnProjectPKPNG)
                    bnNodesPnc.encodeAbstract(
                        paccessor.paGetBnNodes(conn));
                jorm.BnInitiatorMapperPKPNG png8 =
                    (jorm.BnInitiatorMapperPKPNG)
                    bnInitiatorMapperPnc.encodeAbstract(
                        paccessor.paGetBnInitiatorMapper(conn));
                jorm.BnProjectPKPNG png3 =
                    (jorm.BnProjectPKPNG)
                    bnAgentsPnc.encodeAbstract(
                        paccessor.paGetBnAgents(conn));
                jorm.BnProjectPKPNG png11 =
                    (jorm.BnProjectPKPNG)
                    bnUsersPnc.encodeAbstract(
                        paccessor.paGetBnUsers(conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnProject SET creator = ?, enddate = ?, type = ?, BnInitiatorMapper_id = ?, state = ?, status = ?, name = ?, id = ?, parent = ?, creationdate = ? WHERE id = ?");
				//creator
                nullField = paccessor.paGetCreator();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//enddate
                nullField = paccessor.paGetEndDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 2, ((java.util.Date) nullField), null);
                }
				//type
                nullField = paccessor.paGetType();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//BnInitiatorMapper_id
                nullField = png8.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 5, paccessor.paGetState());
				//status
                nullField = paccessor.paGetStatus();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//parent
                nullField = paccessor.paGetParent();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 10, ((java.util.Date) nullField), null);
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnInterHooks_NAME, conn));
                jorm.BnProjectPKPNG png4 =
                    (jorm.BnProjectPKPNG)
                    bnEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnEdges_NAME, conn));
                jorm.BnProjectPKPNG png10 =
                    (jorm.BnProjectPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProperties_NAME, conn));
                jorm.BnProjectPKPNG png2 =
                    (jorm.BnProjectPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnRoles_NAME, conn));
                jorm.BnProjectPKPNG png7 =
                    (jorm.BnProjectPKPNG)
                    bnIterationsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnIterations_NAME, conn));
                jorm.BnProjectPKPNG png1 =
                    (jorm.BnProjectPKPNG)
                    bnAgentEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnAgentEdges_NAME, conn));
                jorm.BnProjectPKPNG png9 =
                    (jorm.BnProjectPKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnHooks_NAME, conn));
                jorm.BnProjectPKPNG png6 =
                    (jorm.BnProjectPKPNG)
                    bnNodesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnNodes_NAME, conn));
                jorm.BnInitiatorMapperPKPNG png8 =
                    (jorm.BnInitiatorMapperPKPNG)
                    bnInitiatorMapperPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnInitiatorMapper_NAME, conn));
                jorm.BnProjectPKPNG png3 =
                    (jorm.BnProjectPKPNG)
                    bnAgentsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnAgents_NAME, conn));
                jorm.BnProjectPKPNG png11 =
                    (jorm.BnProjectPKPNG)
                    bnUsersPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnUsers_NAME, conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnProject SET creator = ?, enddate = ?, type = ?, BnInitiatorMapper_id = ?, state = ?, status = ?, name = ?, id = ?, parent = ?, creationdate = ? WHERE id = ?");
				//creator
                nullField = paccessorGen.paGetStringField(creator_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//enddate
                nullField = paccessorGen.paGetDateField(endDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 2, ((java.util.Date) nullField), null);
                }
				//type
                nullField = paccessorGen.paGetStringField(type_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//BnInitiatorMapper_id
                nullField = png8.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 5, paccessorGen.paGetIntField(state_NAME));
				//status
                nullField = paccessorGen.paGetStringField(status_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessorGen.paGetStringField(name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//parent
                nullField = paccessorGen.paGetStringField(parent_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 10, ((java.util.Date) nullField), null);
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertBnProjectTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                BnProjectAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnProjectPKPNG _pngId =
                (jorm.BnProjectPKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessor.paGetBnInterHooks(conn));
                jorm.BnProjectPKPNG png4 =
                    (jorm.BnProjectPKPNG)
                    bnEdgesPnc.encodeAbstract(
                        paccessor.paGetBnEdges(conn));
                jorm.BnProjectPKPNG png10 =
                    (jorm.BnProjectPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessor.paGetBnProperties(conn));
                jorm.BnProjectPKPNG png2 =
                    (jorm.BnProjectPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessor.paGetBnRoles(conn));
                jorm.BnProjectPKPNG png7 =
                    (jorm.BnProjectPKPNG)
                    bnIterationsPnc.encodeAbstract(
                        paccessor.paGetBnIterations(conn));
                jorm.BnProjectPKPNG png1 =
                    (jorm.BnProjectPKPNG)
                    bnAgentEdgesPnc.encodeAbstract(
                        paccessor.paGetBnAgentEdges(conn));
                jorm.BnProjectPKPNG png9 =
                    (jorm.BnProjectPKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessor.paGetBnHooks(conn));
                jorm.BnProjectPKPNG png6 =
                    (jorm.BnProjectPKPNG)
                    bnNodesPnc.encodeAbstract(
                        paccessor.paGetBnNodes(conn));
                jorm.BnInitiatorMapperPKPNG png8 =
                    (jorm.BnInitiatorMapperPKPNG)
                    bnInitiatorMapperPnc.encodeAbstract(
                        paccessor.paGetBnInitiatorMapper(conn));
                jorm.BnProjectPKPNG png3 =
                    (jorm.BnProjectPKPNG)
                    bnAgentsPnc.encodeAbstract(
                        paccessor.paGetBnAgents(conn));
                jorm.BnProjectPKPNG png11 =
                    (jorm.BnProjectPKPNG)
                    bnUsersPnc.encodeAbstract(
                        paccessor.paGetBnUsers(conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnProject (creator, enddate, type, BnInitiatorMapper_id, state, status, name, id, parent, creationdate) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				//creator
                nullField = paccessor.paGetCreator();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//enddate
                nullField = paccessor.paGetEndDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 2, ((java.util.Date) nullField), null);
                }
				//type
                nullField = paccessor.paGetType();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//BnInitiatorMapper_id
                nullField = png8.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 5, paccessor.paGetState());
				//status
                nullField = paccessor.paGetStatus();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//parent
                nullField = paccessor.paGetParent();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 10, ((java.util.Date) nullField), null);
                }
            } else if (paccessorGen != null) {
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnInterHooks_NAME, conn));
                jorm.BnProjectPKPNG png4 =
                    (jorm.BnProjectPKPNG)
                    bnEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnEdges_NAME, conn));
                jorm.BnProjectPKPNG png10 =
                    (jorm.BnProjectPKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProperties_NAME, conn));
                jorm.BnProjectPKPNG png2 =
                    (jorm.BnProjectPKPNG)
                    bnRolesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnRoles_NAME, conn));
                jorm.BnProjectPKPNG png7 =
                    (jorm.BnProjectPKPNG)
                    bnIterationsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnIterations_NAME, conn));
                jorm.BnProjectPKPNG png1 =
                    (jorm.BnProjectPKPNG)
                    bnAgentEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnAgentEdges_NAME, conn));
                jorm.BnProjectPKPNG png9 =
                    (jorm.BnProjectPKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnHooks_NAME, conn));
                jorm.BnProjectPKPNG png6 =
                    (jorm.BnProjectPKPNG)
                    bnNodesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnNodes_NAME, conn));
                jorm.BnInitiatorMapperPKPNG png8 =
                    (jorm.BnInitiatorMapperPKPNG)
                    bnInitiatorMapperPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnInitiatorMapper_NAME, conn));
                jorm.BnProjectPKPNG png3 =
                    (jorm.BnProjectPKPNG)
                    bnAgentsPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnAgents_NAME, conn));
                jorm.BnProjectPKPNG png11 =
                    (jorm.BnProjectPKPNG)
                    bnUsersPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnUsers_NAME, conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnProject (creator, enddate, type, BnInitiatorMapper_id, state, status, name, id, parent, creationdate) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				//creator
                nullField = paccessorGen.paGetStringField(creator_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//enddate
                nullField = paccessorGen.paGetDateField(endDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 2, ((java.util.Date) nullField), null);
                }
				//type
                nullField = paccessorGen.paGetStringField(type_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//BnInitiatorMapper_id
                nullField = png8.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 5, paccessorGen.paGetIntField(state_NAME));
				//status
                nullField = paccessorGen.paGetStringField(status_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 6, ((java.lang.String) nullField));
                }
				//name
                nullField = paccessorGen.paGetStringField(name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 8, ((java.lang.String) nullField));
                }
				//parent
                nullField = paccessorGen.paGetStringField(parent_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 9, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 10, ((java.util.Date) nullField), null);
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteBnProjectTable(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	BnProjectAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM BnProject WHERE id = ?");
        jorm.BnProjectPKPNG _pngId =
            (jorm.BnProjectPKPNG)
            pname.encodeAbstract();
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("BnProject");
		JoinedTable.Join jtj = null;
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"BnProject.BnInitiatorMapper_id", "BnProject.creationdate", "BnProject.creator", "BnProject.enddate", "BnProject.id", "BnProject.name", "BnProject.parent", "BnProject.state", "BnProject.status", "BnProject.type"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("id", "BnProject.id");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("BnProject.BnInitiatorMapper_id, BnProject.creationdate, BnProject.creator, BnProject.enddate, BnProject.id, BnProject.name, BnProject.parent, BnProject.state, BnProject.status, BnProject.type",
			Collections.singletonList(getJoinedTable()),
			"BnProject.id = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("BnProject.id",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("BnProject.BnInitiatorMapper_id, BnProject.creationdate, BnProject.creator, BnProject.enddate, BnProject.id, BnProject.name, BnProject.parent, BnProject.state, BnProject.status, BnProject.type, BnProject.id" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeAbstract(png, null);
	}



	
    // IMPLEMENTATION OF THE SPECIFIC PNAME GETTER
    public java.lang.String pnGetId(Object ctx)
            throws PException {
	    BnProjectAccessor paccessor = null;
		if (!(ctx instanceof BnProjectAccessor)) {
			throw new PExceptionProtocol("Unsupported context type: " + ctx);
		}
        try {
            return ((BnProjectAccessor) ctx).paGetId();
        } catch (Exception e) {
		    throw new PException(e, "Cannot read the field 'id'");
        }
    }
    // IMPLEMENTATION OF THE PNameGetter INTERFACE

    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
		if ("id".equals(fn)) {
			if (ctx instanceof BnProjectAccessor) {
				return ((BnProjectAccessor) ctx).paGetId();
			} else {
			    throw new PExceptionProtocol("Unsupported context type: " + ctx);
			}
        }
        throw new PException("Cannot read the field " + fn);
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
		((Loggable) bnAgentEdgesGCM).setLogger(l);
		((Loggable) bnAgentsGCM).setLogger(l);
		((Loggable) bnEdgesGCM).setLogger(l);
		((Loggable) bnHooksGCM).setLogger(l);
		((Loggable) bnInterHooksGCM).setLogger(l);
		((Loggable) bnIterationsGCM).setLogger(l);
		((Loggable) bnNodesGCM).setLogger(l);
		((Loggable) bnPropertiesGCM).setLogger(l);
		((Loggable) bnRolesGCM).setLogger(l);
		((Loggable) bnUsersGCM).setLogger(l);
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}


/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG5
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG5(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG4
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG4(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG10
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG10(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG2
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG2(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG7
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG7(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG1
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG1(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG9
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG9(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG6
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG6(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG8
    implements PNameGetter, jorm.BnInitiatorMapperPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG8(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnInitiatorMapperPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 1, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 1, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG3
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG3(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnProjectPNG11
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnProjectPNG11(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 5, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 5, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnAgentEdgesPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnAgentEdgesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnAgentsPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnAgentsPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnEdgesPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnEdgesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnHooksPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnHooksPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnInitiatorMapperPNGPrefetch
        implements PNameGetter, jorm.BnInitiatorMapperPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnInitiatorMapperPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnInitiatorMapperPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(1)) {
                    return ((Tuple) ctx).getString(1);
                } else {
                    return ((jorm.BnInitiatorMapperPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(1)) {
                        return ((Tuple) ctx).getString(1);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnInterHooksPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnInterHooksPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnIterationsPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnIterationsPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnNodesPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnNodesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnPropertiesPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnPropertiesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnRolesPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnRolesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnProject_bnUsersPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnProject_bnUsersPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(5)) {
                    return ((Tuple) ctx).getString(5);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(5)) {
                        return ((Tuple) ctx).getString(5);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }



class BnProjectResultIterator implements PNameIterator, jorm.BnProjectPKPNG {
    ResultSet rs;
	private PBinder binder;
	private BnProjectMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	BnProjectResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		BnProjectMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, -1, this);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }

    // IMPLEMENTATION OF THE BnProjectPKPNG INTERFACE //

    public java.lang.String pnGetId(Object ctx) throws PExceptionIO, PExceptionTyping {
        try {
            return pclassMapping.adapter.getString(rs, pnameIndex + 0, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Impossible to fetch a the field id of the pname");
        }
    }
}
